//
//  Shader_ES300_Base.vsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 23/06/2014.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//
//------------------------------------------------------------------------------

// Defines and macros to allow the shader code to be compiled for ES2 and ES3...
#if defined ES_300
#define ATTRIBUTE				in
#define VARYING_IN				in
#define VARYING_OUT				out
#define UNIFORM					uniform
#define SAMPLER2D				uniform sampler2D
#define SAMPLER2DSHADOW			uniform mediump sampler2DShadow
#define TEXTURE2D(a,b)			texture(a,b)
#define TEXTURE2DBIAS(a,b,c)	texture(a,b,c)
#define TEXTURE2DSHADOW(a,b)	texture(a,b.xyz/b.w)
#define FRAGCOLOUR				out_fragColour
#else
#define ATTRIBUTE				attribute
#define VARYING_IN				varying
#define VARYING_OUT				varying
#define UNIFORM					uniform
#define SAMPLER2D				uniform sampler2D
#define SAMPLER2DSHADOW			uniform sampler2D
#define TEXTURE2D(a,b)			texture2D(a,b)
#define TEXTURE2DBIAS(a,b,c)	texture2D(a,b,c)
#define TEXTURE2DSHADOW(a,b)	texture2DProj(a,b).x
#define FRAGCOLOUR				gl_FragColor
#endif

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR					(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN				(256.0)	//(16.0)

#define DECOMPRESSION_MULTIPLIER_POSITION		(1.0 / POS_COMPRESSION_FACTOR)
#define DECOMPRESSION_MULTIPLIER_POSITION_SKIN	(1.0 / POS_COMPRESSION_FACTOR_SKIN)
#define DECOMPRESSION_MULTIPLIER_UV				(1.0 / 1024.0)
#define DECOMPRESSION_MULTIPLIER_WEIGHT			(1.0 / 255.0)

//------------------------------------------------------------------------------

#define LIGHT_UWFADE_START		(0.0)
#define LIGHT_UWFADE_END		(-40.0)

//------------------------------------------------------------------------------

#define CAUSTICS_FADE_START1	(-12.0)
#define CAUSTICS_FADE_END1		(0.0)

#define CAUSTICS_FADE_START2	(-5.0)
#define CAUSTICS_FADE_END2		(0.0)

#define CAUSTICS_UV_MULTIPLIER	(1.0 / 256.0)

//------------------------------------------------------------------------------

#define UWTINT_START			(0.0)
#define UWTINT_END				(-6.0)

//------------------------------------------------------------------------------

#define UW_RGB_SCALE			(0.7)

//------------------------------------------------------------------------------

ATTRIBUTE highp	vec4 attrib_position;

#if defined (SHADER_LIGHT)
ATTRIBUTE highp	vec3 attrib_normal;
#endif

#if defined (SHADER_TEXTURE)
ATTRIBUTE highp	vec2 attrib_uv0;
#if defined (SHADER_TEXTURE_MULTI_UV2)
ATTRIBUTE highp	vec2 attrib_uv1;
#endif
#endif

#if defined (SHADER_VCOL)
ATTRIBUTE highp	vec4 attrib_rgba;
#endif

#if defined (SHADER_SKIN)
ATTRIBUTE highp	vec4 attrib_boneWeights;
ATTRIBUTE highp	vec4 attrib_boneIndices;
#endif

//------------------------------------------------------------------------------

UNIFORM highp	mat4 uniform_modelViewProjectionMatrix;

#if defined (SHADER_LIGHT)
UNIFORM highp	mat4 uniform_lightDirectionMat;
#endif
#if defined (SHADER_LIGHT) || defined (SHADER_SHADOW_RECEIVE)
UNIFORM highp	mat4 uniform_lightColourMat;
#endif

#if defined (SHADER_FIXED_COLOUR)
UNIFORM highp	vec4 uniform_colour;
#endif

#if defined (SHADER_TEXTURE_ANIMUV) || defined (SHADER_TEXTURE_UWCAUSTICS)
UNIFORM highp	vec4 uniform_animUV;
#endif

#if defined (SHADER_TEXTURE_UVOFFSET)
UNIFORM highp	vec2 uniform_uvOffset;
#endif

#if defined (SHADER_SKIN)
#if defined (ANDROID)
UNIFORM highp	vec4 uniform_boneMat[32*4];	//LM: Some Android devices cannot index matrix arrays by variable...
#else
UNIFORM highp	mat4 uniform_boneMat[32];
#endif
#endif

#if defined (SHADER_SHADOW_RECEIVE) || defined (SHADER_UWTINT)
#if !defined (SHADER_SKIN)
UNIFORM highp	mat4 uniform_modelMatrix;
#endif
#endif

#if defined (SHADER_SHADOW_RECEIVE)
UNIFORM highp	mat4 uniform_shadowProjectionMatrix;
#endif

#if defined (SHADER_UWTINT)
UNIFORM mediump	vec4 uniform_uwTintColour;
#endif

//------------------------------------------------------------------------------

#if defined (SHADER_LIGHT) || defined (SHADER_VCOL) || defined (SHADER_FIXED_COLOUR)
VARYING_OUT lowp	vec4 varying_rgba;
#endif

#if defined (SHADER_TEXTURE)
VARYING_OUT lowp	vec2 varying_uv0;
#if defined (SHADER_TEXTURE_MULTI_UV2)
VARYING_OUT lowp	vec2 varying_uv1;
#endif
#if defined (SHADER_TEXTURE_UWCAUSTICS)
VARYING_OUT mediump	vec4 varying_causticsUVandFadeValues;
#endif
#endif

#if defined (SHADER_UWTINT)
VARYING_OUT mediump	vec4 varying_uwTintColour;
#endif

#if defined (SHADER_SHADOW_RECEIVE)
VARYING_OUT lowp	vec4 varying_shadowAmbient;
VARYING_OUT highp	vec4 varying_shadowUVZW;
#endif

//------------------------------------------------------------------------------

void main()
{
	//--------------------------------------------------------------------------
    // Position...
	//--------------------------------------------------------------------------
#if defined (SHADER_SKIN)
	vec3	posScaled	= (attrib_position.xyz * DECOMPRESSION_MULTIPLIER_POSITION_SKIN);
	vec4	posScaled4	= vec4(posScaled, 1.0);
	
	vec4	weights		= (attrib_boneWeights * DECOMPRESSION_MULTIPLIER_WEIGHT);
	
	//LM: Some Android devices cannot index matrix arrays by variable...
#if defined (ANDROID)
	int		bone0Index	= int(attrib_boneIndices.x) * 4;
	int		bone1Index	= int(attrib_boneIndices.y) * 4;
	mat4	bone0Mat3 = mat4(uniform_boneMat[bone0Index+0], uniform_boneMat[bone0Index+1], uniform_boneMat[bone0Index+2], uniform_boneMat[bone0Index+3]);
	mat4	bone1Mat3 = mat4(uniform_boneMat[bone1Index+0], uniform_boneMat[bone1Index+1], uniform_boneMat[bone1Index+2], uniform_boneMat[bone1Index+3]);
	#if defined (SHADER_SKIN_4BONE)
	int		bone2Index	= int(attrib_boneIndices.z) * 4;
	int		bone3Index	= int(attrib_boneIndices.w) * 4;
	mat4	bone2Mat3 = mat4(uniform_boneMat[bone2Index+0], uniform_boneMat[bone2Index+1], uniform_boneMat[bone2Index+2], uniform_boneMat[bone2Index+3]);
	mat4	bone3Mat3 = mat4(uniform_boneMat[bone3Index+0], uniform_boneMat[bone3Index+1], uniform_boneMat[bone3Index+2], uniform_boneMat[bone3Index+3]);
	#endif
#else
	mat4	bone0Mat3	= uniform_boneMat[int(attrib_boneIndices.x)];
	mat4	bone1Mat3	= uniform_boneMat[int(attrib_boneIndices.y)];
	#if defined (SHADER_SKIN_4BONE)
	mat4	bone2Mat3	= uniform_boneMat[int(attrib_boneIndices.z)];
	mat4	bone3Mat3	= uniform_boneMat[int(attrib_boneIndices.w)];
	#endif
#endif
	
	vec4	posSkin		= ((bone0Mat3 * posScaled4) * weights.x);
			posSkin    += ((bone1Mat3 * posScaled4) * weights.y);
	#if defined (SHADER_SKIN_4BONE)
			posSkin    += ((bone2Mat3 * posScaled4) * weights.z);
			posSkin    += ((bone3Mat3 * posScaled4) * weights.w);
	#endif
	
			posScaled4	= posSkin;

#else
	vec3	posScaled	= (attrib_position.xyz * DECOMPRESSION_MULTIPLIER_POSITION);
	vec4	posScaled4	= vec4(posScaled, 1.0);
#endif

	gl_Position			= (uniform_modelViewProjectionMatrix * posScaled4);

	//--------------------------------------------------------------------------
    // Texture coordinates...
	//--------------------------------------------------------------------------
#if defined (SHADER_TEXTURE)
	// SPECIAL: Faded animated caustic effect based on depth below water...
#if defined (SHADER_TEXTURE_UWCAUSTICS)

	varying_uv0							= (attrib_uv0 * DECOMPRESSION_MULTIPLIER_UV);

	mediump	vec2	baseUVOffset		= fract(uniform_animUV.xy * uniform_animUV.zw);
	mediump vec2	causticsUV			= vec2(-posScaled.x, posScaled.z) * 1.2;
					causticsUV			= ((causticsUV * CAUSTICS_UV_MULTIPLIER) + baseUVOffset);
		
	mediump float	causticsFade1		= ((posScaled.y - CAUSTICS_FADE_START1) * (1.0 / (CAUSTICS_FADE_END1 - CAUSTICS_FADE_START1)));
	mediump float	causticsFade2		= ((posScaled.y - CAUSTICS_FADE_START2) * (1.0 / (CAUSTICS_FADE_END2 - CAUSTICS_FADE_START2)));
	
	causticsFade1 = clamp(causticsFade1, 0.0, 1.0);
	causticsFade2 = clamp(causticsFade2, 0.0, 1.0);
	
	varying_causticsUVandFadeValues		= vec4(causticsUV, causticsFade1, causticsFade2);

#elif defined (SHADER_TEXTURE_ANIMUV)
	
	mediump	vec2	baseUVOffset = fract(uniform_animUV.xy * uniform_animUV.zw);
	
	varying_uv0			= ((attrib_uv0 * DECOMPRESSION_MULTIPLIER_UV) + baseUVOffset);
	
	#if defined (SHADER_TEXTURE_MULTI_UV2)
	varying_uv1			= ((attrib_uv1 * DECOMPRESSION_MULTIPLIER_UV) + baseUVOffset);
	#endif
	
#else
	
	varying_uv0			= (attrib_uv0 * DECOMPRESSION_MULTIPLIER_UV);
	
	#if defined (SHADER_TEXTURE_MULTI_UV2)
	varying_uv1			= (attrib_uv1 * DECOMPRESSION_MULTIPLIER_UV);
	#endif
	
#endif
	
#if defined (SHADER_TEXTURE_UVOFFSET)
	varying_uv0			+= uniform_uvOffset;
#endif
#endif
	
	//--------------------------------------------------------------------------
    // Lighting...
	//--------------------------------------------------------------------------
#if defined (SHADER_LIGHT)
	
	#if defined (SHADER_SKIN)
	
		vec3	normalSkin	= ((mat3(bone0Mat3) * attrib_normal) * weights.x);
				normalSkin += ((mat3(bone1Mat3) * attrib_normal) * weights.y);
		#if defined (SHADER_SKIN_4BONE)
				normalSkin += ((mat3(bone2Mat3) * attrib_normal) * weights.z);
				normalSkin += ((mat3(bone3Mat3) * attrib_normal) * weights.w);
		#endif
	
		// Normalize the skinned normal to prevent scale in the bone matrices affecting the lighting...
		normalSkin = normalize(normalSkin);
	
		// Create the dot products for the three direction lights.
		vec4	normalLightDots	= (uniform_lightDirectionMat * vec4(normalSkin, 0.0));
	
	#else
	
		// Create the dot products for the three direction lights.
		vec4	normalLightDots	= (uniform_lightDirectionMat * vec4(attrib_normal, 0.0));
	
	#endif

    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
    normalLightDots		= clamp(normalLightDots, 0.0, 1.0);
#if defined (SHADER_LIGHT_UWFADE)
	normalLightDots.zw	= vec2(0.0, 1.0);
#else
	normalLightDots.w	= 1.0;
#endif
	
    // Apply the dots to the colour matrix and copy to the output.
	#if defined (SHADER_VCOL)
    varying_rgba		= ((uniform_lightColourMat * normalLightDots) * attrib_rgba);
	#elif defined (SHADER_FIXED_COLOUR)
    varying_rgba		= ((uniform_lightColourMat * normalLightDots) * uniform_colour);
	#else
    varying_rgba		= (uniform_lightColourMat * normalLightDots);
	#endif
	
#elif defined (SHADER_VCOL)
	
	#if defined (SHADER_FIXED_COLOUR)
	varying_rgba		= (attrib_rgba * uniform_colour);
	#else
	varying_rgba		= attrib_rgba;
	#endif
	
#elif defined (SHADER_FIXED_COLOUR)

	varying_rgba		= uniform_colour;

#endif

	//--------------------------------------------------------------------------
    // Shadows...
	//--------------------------------------------------------------------------
#if defined (SHADER_SHADOW_RECEIVE) || defined (SHADER_UWTINT)
#if defined (SHADER_SKIN)
	highp vec4	posWorld	= posScaled4;
#else
	highp vec4	posWorld	= (uniform_modelMatrix * posScaled4);
#endif
#endif

#if defined (SHADER_SHADOW_RECEIVE)
	varying_shadowAmbient	= (varying_rgba - (uniform_lightColourMat[0].xyzw * normalLightDots.x * 0.75));
	varying_shadowUVZW		= (uniform_shadowProjectionMatrix * posWorld);
#endif

	// SPECIAL: Colour tint based on depth below water...
#if defined (SHADER_UWTINT)
	mediump	float	fogValue = ((posWorld.y - UWTINT_START) * (1.0 / (UWTINT_END - UWTINT_START)));
	varying_uwTintColour = vec4(uniform_uwTintColour.xyz, fogValue);
#endif

	// SPECIAL: Fade lighting effect based on depth below water...
#if defined (SHADER_LIGHT)
#if defined (SHADER_LIGHT_UWFADE)
	mediump	float	fogValue = ((posScaled.y - LIGHT_UWFADE_START) * (1.0 / (LIGHT_UWFADE_END - LIGHT_UWFADE_START)));
	fogValue = clamp(fogValue, 0.0, 1.0);
	mediump	float	oneMinusFog = (1.0 - fogValue);
	// Tint the deeper underwater colour using the third directional light colour...
	mediump	vec3	uw_colour = ((uniform_lightColourMat[2].xyz * fogValue) + oneMinusFog);
	varying_rgba.xyz = (((varying_rgba.xyz * oneMinusFog) + fogValue) * UW_RGB_SCALE) * uw_colour;
#if defined (SHADER_SHADOW_RECEIVE)
	varying_shadowAmbient.xyz = (((varying_shadowAmbient.xyz * (1.0 - fogValue)) + fogValue) * UW_RGB_SCALE) * uw_colour;
#endif
#endif
#endif
}

//------------------------------------------------------------------------------
