//
//  Shader_LightTex.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)
#define UV_DECOMPRESSION_MULTIPLIER			(1.0/1024.0)

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec3 normal;
attribute vec4 color;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4 modelViewProjectionMatrix;
uniform highp	mat4 modelMatrix;
uniform highp	vec3 eyePosition;
uniform highp	vec4 animUV;
uniform mediump	vec4 colour;

//------------------------------------------------------------------------------

varying lowp vec4 colorVarying;
varying mediump vec2 texCoordVarying;
varying mediump	vec3 normalVarying;
varying mediump	vec3 eyeDirVarying;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    mediump vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
    gl_Position = modelViewProjectionMatrix * vec4(posScaled, 1.0);
    //gl_Position = modelViewProjectionMatrix * position;
	
    // Decompress the UVs
    //texCoordVarying = texCoord;
	//    texCoordVarying = (texCoord * UV_DECOMPRESSION_MULTIPLIER);
	mediump	vec2	baseUVOffset	= fract(animUV.xy * animUV.zw);
	texCoordVarying.xy				= ((texCoord * UV_DECOMPRESSION_MULTIPLIER) + (baseUVOffset * vec2(4.0, -6.0)));
	
    colorVarying = (colour * color);
	
	normalVarying = (modelMatrix * vec4(normal, 0.0)).xyz;
	
	highp vec4 posWorld = (modelMatrix * vec4(posScaled, 1.0));
	mediump vec3 eyeToPos = (posWorld.xyz - eyePosition.xyz);
	
	eyeDirVarying = normalize(eyeToPos);
}

//------------------------------------------------------------------------------

