//
//  Shader_PreLitTexReflectiveWater.fsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 11/06/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#if defined (ANDROID)
#define TEXTURE2DBIAS(a,b,c)	texture2D(a,b)
#else
#define TEXTURE2DBIAS(a,b,c)	texture2D(a,b,c)
#endif

//#define TRANSITION_VERSION

//------------------------------------------------------------------------------

uniform sampler2D		texture;
uniform sampler2D		normalmap;
uniform sampler2D		reflectionmap;

#if defined (TRANSITION_VERSION)
uniform sampler2D		texture2;
uniform sampler2D		texture3;
#endif

//------------------------------------------------------------------------------

varying mediump vec2	varying_TexCoord;
varying mediump	vec2	varying_NormUV1;
varying mediump	vec2	varying_NormUV2;
varying mediump	vec3	varying_V;
//varying mediump	vec3	varying_L;
varying mediump	vec3	varying_ViewUp;
varying mediump	vec3	varying_ViewRight;

#if defined (TRANSITION_VERSION)
varying mediump vec2	varying_TexCoord2;
#endif

//------------------------------------------------------------------------------

void main()
{
	mediump	vec4	normal0Texel	= TEXTURE2DBIAS(normalmap, varying_NormUV1, -1.0);
	mediump	vec4	normal1Texel	= TEXTURE2DBIAS(normalmap, varying_NormUV2, -1.0);
	
#if defined (TRANSITION_VERSION)
	mediump	vec4	diffuseTexel	= mix(texture2D(texture, varying_TexCoord), texture2D(texture2, varying_TexCoord), texture2D(texture3, varying_TexCoord2));
#else
	mediump	vec4	diffuseTexel	= texture2D(texture, varying_TexCoord);
#endif
	
	mediump	vec3	Vn				= normalize(varying_V);
//	mediump	vec3	Ln				= normalize(varying_L);
	mediump	vec3	Ln				= vec3(0.57735, 0.57735, -0.57735);
	
	mediump	vec3	normal0			= ((2.0 * normal0Texel.xyz) - 1.0);
	mediump	vec3	normal1			= ((2.0 * normal1Texel.xyz) - 1.0);
	mediump	vec3	normal2			= min(vec3(1.0, 1.0, 1.0), (normal0 + normal1));
//	mediump	vec3	normal2			= (normal0 + normal1);
	
	mediump float	flatness		= clamp((diffuseTexel.y - 0.5), 0.0, 1.0);
	normal2.z += (flatness * 8.0);
	
	mediump	vec3	Nn				= normalize(vec3(normal2.y, (normal2.z + 1.0), -normal2.x));	// Equivalent to combining 2 normal maps in Blender with a strength of 0.5 each.
//	mediump	vec3	Nn				= vec3(0.0, 1.0, 0.0);
	
	mediump	float	NdotL			= dot(Nn, Ln);
	
	// Phong specular...
	mediump	vec3	phongR			= ((2.0 * Nn * NdotL) - Ln);
	mediump	float	specularLight	= pow(max(dot(phongR, Vn), 0.0), 80.0);
	
	// Blinn Phong Specular...
//	mediump	vec3	Hn				= normalize(Ln + Vn);
//	mediump	float	NdotH			= dot(Nn, Hn);
//	mediump	float	specularLight	= pow(max(0.0, NdotH), 36.0);
	
	// Reflection...
	mediump	vec3	RNn				= Nn * 0.5;
	mediump	vec3	R				= -((2.0 * dot(RNn, Vn) * RNn) - Vn);
//	mediump	vec3	Rn				= normalize(R);
//	mediump	vec2	Ruv				= vec2(dot(Rn, varying_ViewRight), dot(Rn, varying_ViewUp));
	mediump	vec2	Ruv				= vec2(dot(R, varying_ViewRight), dot(R, varying_ViewUp));
	mediump	vec2	reflectionUV	= ((Ruv * 0.5) + vec2(0.5, 1.0));
	mediump	vec4	reflection		= texture2D(reflectionmap, reflectionUV);
	
	// Diffuse...
	mediump	float	diffuseLight	= max(0.0, (0.1 + (0.9 * NdotL)));//max(0.0, NdotL);
	
	mediump	vec3	ambientColour	= vec3(0.30, 0.30, 0.30);
	mediump	vec3	diffuseColour	= vec3(0.96, 0.96, 0.96);//vec3(0.8, 0.8, 0.8);
	mediump	vec3	specularColour	= vec3(0.35, 0.35, 0.35);//vec3(0.5, 0.5, 0.5);//vec3(1.0, 1.0, 1.0);
	mediump	vec3	reflectColour	= vec3(0.10, 0.10, 0.10);//vec3(0.2, 0.2, 0.2);//vec3(0.2, 0.2, 0.2);

	mediump	vec3	finalDiffuse	= ((ambientColour + diffuseLight) * ((diffuseTexel.xyz * diffuseColour) + (reflection.xyz * reflectColour)));
	mediump	vec3	finalSpecular	= (specularLight * specularColour);
	
	mediump	vec3	finalRGB		= (finalDiffuse + finalSpecular);

    gl_FragColor = vec4(finalRGB, diffuseTexel.w);
}

//------------------------------------------------------------------------------
