//
//  Shader_LightTex.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

//------------------------------------------------------------------------------

#define UWFADE_START		(0.0)
#define UWFADE_END			(8.0)
#define UWFADE_MULTIPLIER	(1.0 / (UWFADE_END - UWFADE_START))

#define UWFADE2_START		(0.0)
#define UWFADE2_END			(32.0)
#define UWFADE2_MULTIPLIER	(1.0 / (UWFADE2_END - UWFADE2_START))

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec3 normal;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4 modelViewProjectionMatrix;
uniform highp	mat4 modelMatrix;
uniform mediump	mat4 lightDirectionMat;
uniform mediump	mat4 lightColorMat;

//------------------------------------------------------------------------------

varying lowp vec4 varying_colour;
varying lowp vec3 varying_fadeValuesAndAboveGround;
varying lowp vec2 texCoordVarying;
varying lowp vec2 varying_screenTexCoord;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
	
	vec4 posScaled4 = vec4(posScaled, 1.0);
	vec4 posTrans = modelViewProjectionMatrix * posScaled4;
	
    gl_Position = posTrans;
	
    // Decompress the UVs
    //texCoordVarying = texCoord;
    texCoordVarying = (texCoord/1024.0);
	
	varying_screenTexCoord.xy = (posTrans.xy / posTrans.w);
	varying_screenTexCoord.xy = ((varying_screenTexCoord.xy.xy * 0.5) + 0.5);
	
    // Create the dot products for the three direction lights.
    vec4 dots = lightDirectionMat * vec4(normal, 0.0);
    
    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
    dots = clamp (dots, 0.0, 1.0);
    dots.w = 1.0;
    
    // Apply the dots to the color matrix and copy to the output.
    mediump vec4 litColour = lightColorMat * dots;
	
	//------------------------------------------------------------------------
	// Simple fogging effect to simulate the characters being underwater...
	//------------------------------------------------------------------------
	
	highp vec4 posWorld = (modelMatrix * vec4(posScaled, 1.0));
	
	mediump	float	fadeValue = ((posWorld.y - UWFADE_START) * UWFADE_MULTIPLIER);
	//					fadeValue = clamp(fadeValue, 0.0, 1.0);
	//	fadeValue = clamp(fadeValue, 0.5, 0.8);
	fadeValue = clamp(fadeValue, 0.6, 1.0);

	mediump	float	fadeValue2 = ((posWorld.y - UWFADE2_START) * UWFADE2_MULTIPLIER);
	fadeValue2 = clamp(fadeValue2, 0.0, 1.0);
	
	varying_colour = litColour;// * vec4(0.0, 0.0, 0.0, 1.0);
	varying_fadeValuesAndAboveGround = vec3(fadeValue, (1.0 - fadeValue2), (posWorld.y - 1.5));	// 2.0 is the water height offset...
}

//------------------------------------------------------------------------------



