//
//  Shader_LightTexSkinned.vsh
//  CityBuilder
//
//  Created by Ben Cohen on 05/04/2012.
//  Copyright (c) 2012 Midoki. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR					(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN				(256.0)	//(16.0)

#define DECOMPRESSION_MULTIPLIER_POSITION		(1.0 / POS_COMPRESSION_FACTOR)
#define DECOMPRESSION_MULTIPLIER_POSITION_SKIN	(1.0 / POS_COMPRESSION_FACTOR_SKIN)
#define DECOMPRESSION_MULTIPLIER_UV				(1.0 / 1024.0)
#define DECOMPRESSION_MULTIPLIER_WEIGHT			(1.0 / 255.0)

//------------------------------------------------------------------------------

#define UWFADE_START		(0.0)
#define UWFADE_END			(8.0)
#define UWFADE_MULTIPLIER	(1.0 / (UWFADE_END - UWFADE_START))

#define UWFADE2_START		(0.0)
#define UWFADE2_END			(32.0)
#define UWFADE2_MULTIPLIER	(1.0 / (UWFADE2_END - UWFADE2_START))

//------------------------------------------------------------------------------

attribute highp vec4 position;
attribute highp vec3 normal;
attribute highp vec2 texCoord;
attribute highp vec4 boneWeight;
attribute highp vec4 boneIndex;

//------------------------------------------------------------------------------

uniform highp mat4 modelViewProjectionMatrix;
uniform highp mat4 lightDirectionMat;
uniform highp mat4 lightColorMat;

#if defined (ANDROID)
uniform highp vec4 boneMat[32*4];	//LM: Some Android devices cannot index matrix arrays by variable...
#else
uniform highp mat4 boneMat[32];
#endif

//------------------------------------------------------------------------------

varying lowp vec4 varying_colour;
varying lowp vec3 varying_fadeValuesAndAboveGround;
varying lowp vec2 texCoordVarying;
varying lowp vec2 varying_screenTexCoord;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
	vec4	posScaled4	= vec4((position.xyz * DECOMPRESSION_MULTIPLIER_POSITION_SKIN), 1.0);

	vec4	weights = (boneWeight * DECOMPRESSION_MULTIPLIER_WEIGHT);
	
#if defined (ANDROID)
	int		bone0Index	= int(boneIndex.x) * 4;
	int		bone1Index	= int(boneIndex.y) * 4;
	//int	bone2Index	= int(boneIndex.z) * 4;
	//int	bone3Index	= int(boneIndex.w) * 4;
	mat4	bone0Mat3 = mat4(boneMat[bone0Index+0], boneMat[bone0Index+1], boneMat[bone0Index+2], boneMat[bone0Index+3]);
	mat4	bone1Mat3 = mat4(boneMat[bone1Index+0], boneMat[bone1Index+1], boneMat[bone1Index+2], boneMat[bone1Index+3]);
	//mat4	bone2Mat3 = mat4(boneMat[bone2Index+0], boneMat[bone2Index+1], boneMat[bone2Index+2], boneMat[bone2Index+3]);
	//mat4	bone3Mat3 = mat4(boneMat[bone3Index+0], boneMat[bone3Index+1], boneMat[bone3Index+2], boneMat[bone3Index+3]);
#else
	mat4	bone0Mat3 = boneMat[int(boneIndex.x)];
	mat4	bone1Mat3 = boneMat[int(boneIndex.y)];
	//mat4	bone2Mat3 = boneMat[int(boneIndex.z)];
	//mat4	bone3Mat3 = boneMat[int(boneIndex.w)];
#endif
	
	vec4	posSkinned  = ((bone0Mat3 * posScaled4) * weights.x);
			posSkinned += ((bone1Mat3 * posScaled4) * weights.y);
//			posSkinned += ((bone2Mat3 * posScaled4) * weights.z);
//			posSkinned += ((bone3Mat3 * posScaled4) * weights.w);
	
	highp vec4 posTrans = (modelViewProjectionMatrix * posSkinned);
	
    gl_Position = posTrans;
	
	vec3	normalSkinned  = ((mat3(bone0Mat3) * normal) * weights.x);
			normalSkinned += ((mat3(bone1Mat3) * normal) * weights.y);
//			normalSkinned += ((mat3(bone2Mat3) * normal) * weights.z);
//			normalSkinned += ((mat3(bone3Mat3) * normal) * weights.w);
	
	// Normalize the skinned normal to prevent scale in the bone matrices affecting the lighting...
	normalSkinned = normalize(normalSkinned);
	
    // Decompress the UVs
    //texCoordVarying = texCoord;
    texCoordVarying = (texCoord * DECOMPRESSION_MULTIPLIER_UV);
	
	varying_screenTexCoord.xy = (posTrans.xy / posTrans.w);
	varying_screenTexCoord.xy = ((varying_screenTexCoord.xy.xy * 0.5) + 0.5);
	
    // Create the dot products for the three direction lights.
    vec4 dots = lightDirectionMat * vec4(normalSkinned, 0.0);
    
    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
    dots = clamp (dots, 0.0, 1.0);
    dots.w = 1.0;
    
    // Apply the dots to the color matrix and copy to the output.
    mediump vec4 litColour = lightColorMat * dots;
	
	//------------------------------------------------------------------------
	// Simple fogging effect to simulate the characters being underwater...
	//------------------------------------------------------------------------
	
	mediump	float	fadeValue = ((posSkinned.y - UWFADE_START) * UWFADE_MULTIPLIER);
	//					fadeValue = clamp(fadeValue, 0.0, 1.0);
	//	fadeValue = clamp(fadeValue, 0.5, 0.8);
	fadeValue = clamp(fadeValue, 0.6, 1.0);
	
	mediump	float	fadeValue2 = ((posSkinned.y - UWFADE2_START) * UWFADE2_MULTIPLIER);
	fadeValue2 = clamp(fadeValue2, 0.0, 1.0);

	varying_colour = litColour;// * vec4(0.0, 0.0, 0.0, 1.0);
	varying_fadeValuesAndAboveGround = vec3(fadeValue, (1.0 - fadeValue2), (posSkinned.y - 1.5));	// 2.0 is the water height offset...
}

//------------------------------------------------------------------------------




