//
//  Shader_PreLitTexReflectiveWater.vsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 11/06/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)
#define UV_DECOMPRESSION_MULTIPLIER			(1.0/1024.0)

#define NORMALMAP_UV_MULTIPLIER1			(1.0/256.0)
#define NORMALMAP_UV_MULTIPLIER2			(1.0/500.0)

#define TRANSITION_VERSION

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec2 texCoord;
attribute vec2 texCoord2;

//------------------------------------------------------------------------------

uniform highp	mat4	modelViewProjectionMatrix;
uniform highp	mat4	modelViewMatrix;

uniform highp	vec4	animUV;

uniform highp	mat4	modelMatrix;
uniform highp	vec3	eyePosition;
uniform highp	vec3	lightPosWorld;

//------------------------------------------------------------------------------

varying mediump vec2	varying_TexCoord;
varying mediump	vec2	varying_NormUV1;
varying mediump	vec2	varying_NormUV2;
varying highp	vec3	varying_V;
//varying highp	vec3	varying_L;
varying mediump	vec3	varying_ViewUp;
varying mediump	vec3	varying_ViewRight;

#if defined (TRANSITION_VERSION)
varying mediump vec2	varying_TexCoord2;
#endif

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
	mediump vec3	posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
	mediump vec4	posScaled4 = vec4(posScaled, 1.0);
	
	mediump	vec3	posWorld		= (modelMatrix * posScaled4).xyz;

	mediump	vec2	texCoordScreen		= vec2(-posWorld.x, posWorld.z) * 0.02;
	
	gl_Position = modelViewProjectionMatrix * posScaled4;
	
    // Decompress UVs
	mediump	vec2	texCoordDecomp	= (texCoord/1024.0);
	
	mediump	vec2	baseUVOffset	= fract(animUV.xy * animUV.zw * vec2(0.75, 0.75));
	mediump	vec2	baseUVOffset2	= fract(animUV.xy * animUV.zw * vec2(2.83, 1.94));
	
	varying_TexCoord				= (texCoordDecomp + baseUVOffset);
	
#if defined (TRANSITION_VERSION)
	mediump	vec2	texCoordDecomp2	= (texCoord2/1024.0);
	varying_TexCoord2				= texCoordDecomp2;
#endif
	
	varying_NormUV1					= -((texCoordScreen * 2.0) + baseUVOffset).yx;
	varying_NormUV2					= -((texCoordScreen * 3.2) + baseUVOffset2).yx;
	
	varying_V						= normalize(eyePosition.xyz - posWorld);
//	varying_L						= (lightPosWorld.xyz - posWorld);
//	varying_L						= normalize(vec3(1.0, 1.0, -1.0));
	
	varying_ViewUp					= vec3(modelViewMatrix[0].y, modelViewMatrix[1].y, modelViewMatrix[2].y);
	varying_ViewRight				= -vec3(modelViewMatrix[0].x, modelViewMatrix[1].x, modelViewMatrix[2].x);
}

//------------------------------------------------------------------------------
