//
//  Shader_LightTexSkinned.vsh
//  CityBuilder
//
//  Created by Ben Cohen on 05/04/2012.
//  Copyright (c) 2012 Midoki. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

//------------------------------------------------------------------------------

#define UWFOG_START			(0.0)
#define UWFOG_END			(-1.0)
#define UWFOG_MULTIPLIER	(1.0 / (UWFOG_END - UWFOG_START))

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec3 normal;
attribute vec2 texCoord;
attribute vec4 boneWeight;
attribute vec4 boneIndex;

//------------------------------------------------------------------------------

uniform mat4 modelViewProjectionMatrix;
uniform mat4 lightDirectionMat;
uniform mat4 lightColorMat;

#if defined (ANDROID)
uniform vec4 boneMat[32*4];	//LM: Some Android devices cannot index matrix arrays by variable...
#else
uniform mat4 boneMat[32];
#endif

//------------------------------------------------------------------------------

varying lowp vec4 varying_colour;
varying lowp vec4 varying_fogColour;
varying lowp vec2 texCoordVarying;
varying lowp vec2 varying_screenTexCoord;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec4	posScaled = vec4(position.xyz * XYZ_DECOMPRESSION_MULTIPLIER_SKIN, 1.0);

	vec4	weights = boneWeight/255.0;
	
#if defined (ANDROID)
	int		bone0Index	= int(boneIndex.x) * 4;
	int		bone1Index	= int(boneIndex.y) * 4;
	//int	bone2Index	= int(boneIndex.z) * 4;
	//int	bone3Index	= int(boneIndex.w) * 4;
	mat4	bone0Mat3 = mat4(boneMat[bone0Index+0], boneMat[bone0Index+1], boneMat[bone0Index+2], boneMat[bone0Index+3]);
	mat4	bone1Mat3 = mat4(boneMat[bone1Index+0], boneMat[bone1Index+1], boneMat[bone1Index+2], boneMat[bone1Index+3]);
	//mat4	bone2Mat3 = mat4(boneMat[bone2Index+0], boneMat[bone2Index+1], boneMat[bone2Index+2], boneMat[bone2Index+3]);
	//mat4	bone3Mat3 = mat4(boneMat[bone3Index+0], boneMat[bone3Index+1], boneMat[bone3Index+2], boneMat[bone3Index+3]);
#else
	mat4	bone0Mat3 = boneMat[int(boneIndex.x)];
	mat4	bone1Mat3 = boneMat[int(boneIndex.y)];
	//mat4	bone2Mat3 = boneMat[int(boneIndex.z)];
	//mat4	bone3Mat3 = boneMat[int(boneIndex.w)];
#endif

	vec4	posSkinned  = ((bone0Mat3 * posScaled) * weights.x);
			posSkinned += ((bone1Mat3 * posScaled) * weights.y);
//			posSkinned += ((bone2Mat3 * posScaled) * weights.z);
//			posSkinned += ((bone3Mat3 * posScaled) * weights.w);
	
	vec3	normalSkinned  = ((mat3(bone0Mat3) * normal) * weights.x);
			normalSkinned += ((mat3(bone1Mat3) * normal) * weights.y);
//			normalSkinned += ((mat3(bone2Mat3) * normal) * weights.z);
//			normalSkinned += ((mat3(bone3Mat3) * normal) * weights.w);

	// Normalize the skinned normal to prevent scale in the bone matrices affecting the lighting...
	normalSkinned = normalize(normalSkinned);

	vec4 posTrans = modelViewProjectionMatrix * posSkinned;

    gl_Position = posTrans;

    // Decompress the UVs
    texCoordVarying = (texCoord/1024.0);

	varying_screenTexCoord.xy = (posTrans.xy / posTrans.w);
	varying_screenTexCoord.xy = ((texCoordVarying.xy * 0.5) + 0.5);

    // Create the dot products for the three direction lights.
    vec4 dots = lightDirectionMat * vec4(normalSkinned, 0.0);
    
    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
    dots = clamp (dots, 0.0, 1.0);
    dots.w = 1.0;
    
    // Apply the dots to the color matrix and copy to the output.
    mediump vec4 litColour = lightColorMat * dots;

	//------------------------------------------------------------------------
	// Simple fogging effect to simulate the characters being underwater...
	//------------------------------------------------------------------------

	mediump	float	fogValue = ((posSkinned.y - UWFOG_START) * UWFOG_MULTIPLIER);
					fogValue = clamp(fogValue, 0.0, 1.0);
	
//	mediump vec3	foggedColour = (vec3(0.0, 0.5, 0.5) * fogValue);
	mediump vec3	foggedColour = (vec3(0.0, 0.4, 0.4) * fogValue);
	
//	litColour *= ((litColour * (1.0 - fogValue)) + vec4(0.5, 0.6, 0.7, 1.0) * fogValue);
	litColour *= ((litColour * (1.0 - fogValue)) + vec4(0.3, 0.5, 0.6, 1.0) * fogValue);
	
	varying_colour = litColour;
	varying_fogColour = vec4(foggedColour, 0.0);
}

//------------------------------------------------------------------------------

