//
//  Shader_LightTex.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)
#define UV_DECOMPRESSION_MULTIPLIER			(1.0/1024.0)

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec3 normal;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4 modelViewProjectionMatrix;
uniform highp	mat4 modelMatrix;
uniform mediump	mat4 lightDirectionMat;
uniform mediump	mat4 lightColorMat;
uniform highp	vec3 eyePosition;
uniform mediump	vec3 lightDirectionWorld;

//------------------------------------------------------------------------------

varying lowp vec4 colorVarying;
varying mediump vec2 texCoordVarying;
varying mediump	vec3 normalVarying;
varying mediump	vec3 refVectorVarying;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    mediump vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
    gl_Position = modelViewProjectionMatrix * vec4(posScaled, 1.0);
    //gl_Position = modelViewProjectionMatrix * position;

    // Decompress the UVs
    //texCoordVarying = texCoord;
    texCoordVarying = (texCoord * UV_DECOMPRESSION_MULTIPLIER);

#if 1
    //colorVarying = vec4 (normal.x, normal.y, normal.z, 1.0);

    // Create the dot products for the three direction lights.
//	mediump vec3 normalN = normalize(normal);
//	mediump vec4 dots = lightDirectionMat * vec4(normalN, 0.0);
	mediump vec4 dots = lightDirectionMat * vec4(normal, 0.0);

    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
//    dots = clamp (dots, 0.0, 1.0);
    dots.xyz = max(dots.xyz, 0.0);
    dots.w = 1.0;
    
    // Apply the dots to the color matrix and copy to the output.
    colorVarying = lightColorMat * dots;
#else
    colorVarying = vec4(1.0, 0.0, 1.0, 1.0);
#endif
	
	normalVarying = (modelMatrix * vec4(normal, 0.0)).xyz;

	highp vec4 posWorld = (modelMatrix * vec4(posScaled, 1.0));
	mediump vec3 eyeToPos = (posWorld.xyz - eyePosition.xyz);
	
	refVectorVarying = (lightDirectionWorld.xyz - normalize(eyeToPos));
}

//------------------------------------------------------------------------------
