//
//  Shader_LightTex.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

//------------------------------------------------------------------------------

#define NORMALMAP_UV_DIVISOR1	(256.0)	//(256.0)

//------------------------------------------------------------------------------

#define UWFOG_STRENGTH		(1.0)
#define UWFOG_START			(0.0)
#define UWFOG_END			(-40.0)
#define UWFOG_MULTIPLIER	(1.0 / (UWFOG_END - UWFOG_START))

//------------------------------------------------------------------------------

#define FADE_START1			(-12.0)
#define FADE_END1			(0.0)
#define FADE_MULTIPLIER1	(1.0 / (FADE_END1 - FADE_START1))
#define FADE_SCALE1			(1.0)

#define FADE_START2			(-5.0)
#define FADE_END2			(0.0)
#define FADE_MULTIPLIER2	(1.0 / (FADE_END2 - FADE_START2))
#define FADE_SCALE2			(1.0)

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec3 normal;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp mat4 modelViewProjectionMatrix;
uniform highp mat4 lightDirectionMat;
uniform highp mat4 lightColorMat;
uniform highp vec4 animUV;

//------------------------------------------------------------------------------

varying lowp	vec4 varying_Colour;
varying lowp	vec4 varying_fogColour;
varying mediump	vec4 varying_TextureUV_FadeValues;
varying mediump	vec2 varying_CausticsUV;

//------------------------------------------------------------------------------

void main()
{
	// Transform the position into screen space.
    vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
    gl_Position = modelViewProjectionMatrix * vec4(posScaled, 1.0);
    //gl_Position = modelViewProjectionMatrix * position;
	
	mediump vec2	baseUVOffset = fract(animUV.xy * animUV.zw);// * 0.4;

    // Decompress UVs
    varying_TextureUV_FadeValues.xy  = (texCoord/1024.0);
 
	mediump vec2	texCoord	= vec2(-posScaled.x, posScaled.z) * 1.2;// * 2.0;//4.0;
	
	varying_CausticsUV = ((texCoord / NORMALMAP_UV_DIVISOR1) + baseUVOffset);

//#if defined (ANDROID)
//	varying_TextureUV.xy  = vec2(varying_TextureUV.x,  (1.0 - varying_TextureUV.y));
//	varying_CausticsUV_FadeValues.xy = vec2(varying_CausticsUV_FadeValue.x, (1.0 - varying_CausticsUV_FadeValue.y));
//#endif
	
#if 1
    //varying_Colour = vec4 (normal.x, normal.y, normal.z, 1.0);
	
    // Create the dot products for the three direction lights.
    vec4 dots = lightDirectionMat * vec4(normal, 0.0);
    
    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
    dots = clamp (dots, 0.0, 1.0);
    dots.w = 1.0;
    
    // Apply the dots to the color matrix and copy to the output.
    lowp vec4	litColour = lightColorMat * dots;
#else
    lowp vec4	litColour = vec4(0.0, 0.0, 1.0, 1.0);
#endif
	
	mediump float fadeValue1 = ((posScaled.y - FADE_START1) * FADE_MULTIPLIER1);
	mediump float fadeValue2 = ((posScaled.y - FADE_START2) * FADE_MULTIPLIER2);
//	fadeValue = clamp((fadeValue * FADE_STRENGTH), 0.0, 1.0);
//	varying_Colour.w = (fadeValue * 0.8);

	fadeValue1 = clamp(fadeValue1, 0.0, 1.0) * FADE_SCALE1;
	fadeValue2 = clamp(fadeValue2, 0.0, 1.0) * FADE_SCALE2;
//	fadeValue = ((fadeValue - 0.5) * 2.0);
	
//	fadeValue = (fadeValue * fadeValue);
//	fadeValue = (1.0 - fadeValue);

	varying_TextureUV_FadeValues.z = fadeValue1;
	varying_TextureUV_FadeValues.w = fadeValue2;

	mediump	float	fogValue = ((posScaled.y - UWFOG_START) * UWFOG_MULTIPLIER);
					fogValue = (fogValue * UWFOG_STRENGTH);
					fogValue = clamp(fogValue, 0.0, 1.0);
	
	mediump vec3	foggedColour = ((litColour.xyz * (1.0 - fogValue)) + fogValue);

	varying_Colour = vec4(foggedColour, 1.0);
}

//------------------------------------------------------------------------------
