//
//  Shader_LightTex.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

//------------------------------------------------------------------------------

#define UWFOG_START				(0.0)
#define UWFOG_END				(-40.0)
#define UWFOG_MULTIPLIER		(1.0 / (UWFOG_END - UWFOG_START))

#define UWBLEND_START			(-10.0)
#define UWBLEND_END				(-20.0)
#define UWBLEND_MULTIPLIER		(1.0 / (UWBLEND_END - UWBLEND_START))

#define TEXTURE2_UV_MULTIPLIER	(1.0 / 1040.0)	//(256.0)	//(256.0)

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec3 normal;
attribute vec2 texCoord;
attribute vec2 texCoord2;

//------------------------------------------------------------------------------

uniform mat4 modelViewProjectionMatrix;
uniform mat4 lightDirectionMat;
uniform mat4 lightColorMat;

//------------------------------------------------------------------------------

varying lowp vec4 varying_colour;
varying lowp vec4 varying_fogColour;
varying lowp vec2 varying_texCoord;
varying lowp vec2 varying_texCoord2;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    mediump vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
    gl_Position = modelViewProjectionMatrix * vec4(posScaled, 1.0);

    // Decompress the UVs
    varying_texCoord  = (texCoord * (1.0 / 1024.0));
//	varying_texCoord2 = (texCoord2/1024.0);

	mediump vec2	texCoordWorld	= vec2(posScaled.x, posScaled.z) * 4.0;
	varying_texCoord2 = (((texCoordWorld * TEXTURE2_UV_MULTIPLIER) * 0.5) + 0.5);
	 

//#if defined (ANDROID)
//	varying_texCoord.xy = vec2(varying_texCoord.x, (1.0 - varying_texCoord.y));
//#endif

    // Create the dot products for the three direction lights.
	mediump vec4	dots = (lightDirectionMat * vec4(normal, 0.0));

    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
    dots = clamp (dots, 0.0, 1.0);
    dots.w = 1.0;
    
    // Apply the dots to the color matrix and copy to the output.
    mediump vec4	litColour = vec4(lightColorMat * dots);
	
//	varying_colour = litColour;

	mediump	vec2	fogValues;
					fogValues.x = ((posScaled.y - UWFOG_START)   * UWFOG_MULTIPLIER);
					fogValues.y = ((posScaled.y - UWBLEND_START) * UWBLEND_MULTIPLIER);
					fogValues   = clamp(fogValues, 0.0, 1.0);

	mediump vec3	foggedColour = ((litColour.xyz * (1.0 - fogValues.x)) + fogValues.x);
	
	varying_colour = vec4(foggedColour, fogValues.y);
}

//------------------------------------------------------------------------------
