//
//  Shader_LightTex.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

//------------------------------------------------------------------------------

#define UWFOG_START			(0.0)
#define UWFOG_END			(-2.0)
#define UWFOG_MULTIPLIER	(1.0 / (UWFOG_END - UWFOG_START))

//------------------------------------------------------------------------------

#define UWFADE_START		(0.0)
#define UWFADE_END			(-3.0)
#define UWFADE_MULTIPLIER	(1.0 / (UWFADE_END - UWFADE_START))

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec3 normal;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4 modelViewProjectionMatrix;
uniform highp	mat4 modelMatrix;
uniform mediump	mat4 lightDirectionMat;
uniform mediump	mat4 lightColorMat;

//------------------------------------------------------------------------------

varying lowp vec4 varying_colour;
varying lowp vec4 varying_fogColour;
varying lowp vec2 texCoordVarying;
varying lowp vec2 varying_screenTexCoord;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
	vec4 posScaled4 = vec4(posScaled, 1.0);
	vec4 posTrans = modelViewProjectionMatrix * posScaled4;
	
    gl_Position = posTrans;

    // Decompress the UVs
    //texCoordVarying = texCoord;
    texCoordVarying = (texCoord/1024.0);

	varying_screenTexCoord.xy = (posTrans.xy / posTrans.w);
	varying_screenTexCoord.xy = ((varying_screenTexCoord.xy.xy * 0.5) + 0.5);

    // Create the dot products for the three direction lights.
    vec4 dots = lightDirectionMat * vec4(normal, 0.0);
    
    // Clamp dots to between 0 and 1 (make sure the ambient dot is 1).
    dots = clamp (dots, 0.0, 1.0);
    dots.w = 1.0;
    
    // Apply the dots to the color matrix and copy to the output.
    mediump vec4 litColour = lightColorMat * dots;

	//------------------------------------------------------------------------
	// Simple fogging effect to simulate the characters being underwater...
	//------------------------------------------------------------------------

	highp vec4 posWorld = (modelMatrix * vec4(posScaled, 1.0));

	mediump	float	fogValue = ((posWorld.y - UWFOG_START) * UWFOG_MULTIPLIER);
					fogValue = clamp(fogValue, 0.0, 1.0);
	
	mediump	float	fadeValue = ((posWorld.y - UWFADE_START) * UWFADE_MULTIPLIER);
					fadeValue = clamp(fadeValue, 0.0, 1.0);
	
//	mediump vec3	foggedColour = (vec3(0.0, 0.5, 0.5) * fogValue);
	mediump vec3	foggedColour = (vec3(0.0, 0.4, 0.4) * fogValue);
	
//	litColour *= ((litColour * (1.0 - fogValue)) + vec4(0.5, 0.6, 0.7, 1.0) * fogValue);
	litColour *= ((litColour * (1.0 - fogValue)) + vec4(0.3, 0.5, 0.6, 1.0) * fogValue);
	
	varying_colour = litColour;
	varying_fogColour = vec4(foggedColour, fadeValue);
}

//------------------------------------------------------------------------------
