//
//  Shader_LightTexSkinned.vsh
//  CityBuilder
//
//  Created by Ben Cohen on 05/04/2012.
//  Copyright (c) 2012 Midoki. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)
#define UV_DECOMPRESSION_MULTIPLIER			(1.0/127.0)//1024.0)

//------------------------------------------------------------------------------

attribute vec4 position;
//attribute vec2 texCoord;
//attribute vec4 boneWeight;
attribute vec4 boneIndex;

//------------------------------------------------------------------------------

uniform mat4 modelViewProjectionMatrix;

#if defined (ANDROID)
uniform vec4 boneMat[32*4];	//LM: Some Android devices cannot index matrix arrays by variable...
#else
uniform mat4 boneMat[32];
#endif

//------------------------------------------------------------------------------

varying lowp vec2 texCoordVarying;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec4	posScaled = vec4(position.xyz * XYZ_DECOMPRESSION_MULTIPLIER_SKIN, 1.0);

	vec2	weights = (boneIndex.zw/255.0);
	
#if defined (ANDROID)
	int		bone0Index	= int(boneIndex.x) * 4;
	//int	bone1Index	= int(boneIndex.y) * 4;
	mat4	bone0Mat3 = mat4(boneMat[bone0Index+0], boneMat[bone0Index+1], boneMat[bone0Index+2], boneMat[bone0Index+3]);
	//mat4	bone1Mat3 = mat4(boneMat[bone1Index+0], boneMat[bone1Index+1], boneMat[bone1Index+2], boneMat[bone1Index+3]);
#else
	mat4	bone0Mat3 = boneMat[int(boneIndex.x)];
	//mat4	bone1Mat3 = boneMat[int(boneIndex.y)];
#endif

	vec4	posSkinned  = ((bone0Mat3 * posScaled) * weights.x);
//			posSkinned += ((bone1Mat3 * posScaled) * weights.y);
	
    gl_Position = modelViewProjectionMatrix * posSkinned;

    // Decompress the UVs
//	texCoordVarying = (texCoord * UV_DECOMPRESSION_MULTIPLIER);
	mediump float v = floor(position.w * (1.0 / 128.0));
	mediump float u = (position.w - (v * 128.0));
	texCoordVarying = vec2((u * (1.0 / 127.0)), (v * (1.0 / 127.0)));
}

//------------------------------------------------------------------------------

