//
//  Shader_LightTexSkinned.vsh
//  CityBuilder
//
//  Created by Ben Cohen on 05/04/2012.
//  Copyright (c) 2012 Midoki. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)
#define UV_DECOMPRESSION_MULTIPLIER			(1.0/127.0)//1024.0)

//------------------------------------------------------------------------------

//#define UWFOG_STRENGTH		(1.0)
#define UWFOG_START			(0.0)
#define UWFOG_END			(-1.0)
#define UWFOG_MULTIPLIER	(1.0 / (UWFOG_END - UWFOG_START))

//------------------------------------------------------------------------------

attribute vec4 position;
//attribute vec2 texCoord;
//attribute vec4 boneWeight;
attribute vec4 boneIndex;

//------------------------------------------------------------------------------

uniform mat4 modelViewProjectionMatrix;
uniform mediump	vec4 uwTintColour;

#if defined (ANDROID)
uniform vec4 boneMat[32*4];	//LM: Some Android devices cannot index matrix arrays by variable...
#else
uniform mat4 boneMat[32];
#endif

//------------------------------------------------------------------------------

varying lowp vec4 varying_colour;
varying lowp vec4 varying_fogColour;
varying lowp vec2 texCoordVarying;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec4	posScaled = vec4(position.xyz * XYZ_DECOMPRESSION_MULTIPLIER_SKIN, 1.0);

	vec2	weights = boneIndex.zw/255.0;
	
#if defined (ANDROID)
	int		bone0Index	= int(boneIndex.x) * 4;
	//int	bone1Index	= int(boneIndex.y) * 4;
	mat4	bone0Mat3 = mat4(boneMat[bone0Index+0], boneMat[bone0Index+1], boneMat[bone0Index+2], boneMat[bone0Index+3]);
	//mat4	bone1Mat3 = mat4(boneMat[bone1Index+0], boneMat[bone1Index+1], boneMat[bone1Index+2], boneMat[bone1Index+3]);
#else
	mat4	bone0Mat3 = boneMat[int(boneIndex.x)];
	//mat4	bone1Mat3 = boneMat[int(boneIndex.y)];
#endif

	vec4	posSkinned  = ((bone0Mat3 * posScaled) * weights.x);
//			posSkinned += ((bone1Mat3 * posScaled) * weights.y);
	
    gl_Position = modelViewProjectionMatrix * posSkinned;

    // Decompress the UVs
//	texCoordVarying = (texCoord * UV_DECOMPRESSION_MULTIPLIER);
	mediump float v = floor(position.w * (1.0 / 128.0));
	mediump float u = (position.w - (v * 128.0));
	texCoordVarying = vec2((u * (1.0 / 127.0)), (v * (1.0 / 127.0)));

//    // Apply the dots to the color matrix and copy to the output.
//    mediump vec4 litColour = vec4(1.0, 1.0, 1.0, 1.0);
//
//	//------------------------------------------------------------------------
//	// Simple fogging effect to simulate the characters being underwater...
//	//------------------------------------------------------------------------
//
//	mediump	float	fogValue = ((posSkinned.y - UWFOG_START) * UWFOG_MULTIPLIER);
////					fogValue = (fogValue * UWFOG_STRENGTH);
//					fogValue = clamp(fogValue, 0.0, 1.0);
//	
////	mediump vec3	foggedColour = (vec3(0.0, 0.5, 0.5) * fogValue);
////	mediump vec3	foggedColour = (vec3(0.0, 0.4, 0.4) * fogValue);
//	mediump vec3	foggedColour = uwTintColour.xyz;//vec3(0.05, 0.55, 0.5);
//	
////	litColour *= ((litColour * (1.0 - fogValue)) + vec4(0.5, 0.6, 0.7, 1.0) * fogValue);
//	litColour *= ((litColour * (1.0 - fogValue)) + vec4(0.3, 0.5, 0.6, 1.0) * fogValue);
//	
//	varying_colour = litColour;
//	varying_fogColour = vec4(foggedColour, fogValue);

	mediump	float	fogValue = sign(max(0.0, -posSkinned.y));
	mediump	float	oneMinusFog = (1.0 - fogValue);
	mediump vec3	foggedColour = (uwTintColour.xyz * fogValue) + vec3(oneMinusFog, oneMinusFog, oneMinusFog);
	
	varying_colour = vec4(foggedColour, 1.0);
}

//------------------------------------------------------------------------------

