//
//  Shader_PreLitTexAnimUV.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

#define COMPRESSED_LIGHT_RATIOS

//------------------------------------------------------------------------------

attribute vec4 position;

#if !defined (COMPRESSED_LIGHT_RATIOS)
attribute vec4 color;
#endif

attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4	modelViewProjectionMatrix;
uniform highp	vec4	modelWorldOffset;

uniform highp	vec4	animUV;

#if defined (COMPRESSED_LIGHT_RATIOS)
uniform lowp	mat4	lightColorMat;
#endif

//------------------------------------------------------------------------------

varying lowp	vec4	colorVarying;
varying highp	vec2	texCoordVarying;	// highp needed to avoid inaccuracies in the anim uv's

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER) + modelWorldOffset.xyz;
    gl_Position = modelViewProjectionMatrix * vec4(posScaled, 1.0);
    //gl_Position = modelViewProjectionMatrix * position;

    // Decompress UVs
    //texCoordVarying = animUV.xy + texCoord;
    texCoordVarying = animUV.xy * animUV.zw + (texCoord/1024.0);

#if defined (COMPRESSED_LIGHT_RATIOS)
	//	highp float ratios = (position.w / 32768.0);
	//	mediump vec3 dots = fract(vec3(1.0, 32.0, 1024.0) * ratios);
	//	colorVarying = (lightColorMat * vec4(dots, 1.0));
	mediump vec3 dots = fract(vec3(1.0/32768.0, 32.0/32768.0, 1024.0/32768.0) * position.w);
	colorVarying = (lightColorMat * vec4(dots, 1.0));
#else
	lowp vec3 rgb = color.xyz/127.0;
	colorVarying = vec4(rgb, 1.0);
#endif
}

//------------------------------------------------------------------------------
