//
//  Shader_PreLitTexReflectiveWater.fsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 11/06/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define REFLECTION
//#define REFLECTION_LATLONG

//------------------------------------------------------------------------------

#define PI 3.1415926

//------------------------------------------------------------------------------

uniform sampler2D texture;

#if defined (REFLECTION)
uniform sampler2D		normalmap;
uniform sampler2D		reflectionmap;
#endif

//------------------------------------------------------------------------------

varying lowp	vec4	colorVarying;

varying highp	vec2	texCoordVarying;

#if defined (REFLECTION)
varying highp	vec2	texCoordVarying2;
varying highp	vec2	texCoordVarying3;
varying mediump	vec3	eyeVectorVarying;
#endif

//------------------------------------------------------------------------------

void main()
{
#if defined (REFLECTION)
	
	mediump	vec3	normal  = ((2.0 * texture2D(normalmap, texCoordVarying2).xyz) - 1.0);
	mediump	vec3	normal2 = ((2.0 * texture2D(normalmap, texCoordVarying3).xyz) - 1.0);

	normal += normal2;
	normal.z *= 32.0;
	normal = normalize(normal);
	//normal = vec3(0,0,1);	// For testing reflections... 
	
	mediump vec3	refl = normalize((2.0 * dot(eyeVectorVarying, normal) * normal) - eyeVectorVarying);

#if defined (REFLECTION_LATLONG)
	// Lat-long...
	mediump float	u = atan(refl.y, refl.x) / (2.0*PI);
	mediump float	xy = sqrt(refl.x*refl.x + refl.y*refl.y);
	mediump float	v = atan(refl.z, xy) / (2.0*PI) + 0.5;
	mediump vec2	uv = vec2(u,v);
#else	
	// Super-fast but hacky use reflection vector xy as uv coordinates...
	//mediump vec2	uv = ((-refl.xy * 1.0) + 0.5);
	mediump vec2	uv = -refl.xy;
	//uv.x = ((uv.x * 1.0) + 0.5);
	uv = ((uv * 0.5) + 0.5);
#endif
	
	mediump vec4 reflection = texture2D(reflectionmap, uv);

//	gl_FragColor = (texture2D(texture, texCoordVarying) * colorVarying * 0.5) + (reflection * 0.7);
	gl_FragColor = (texture2D(texture, texCoordVarying) * colorVarying * 0.6) + (reflection * colorVarying * 0.5);

#else
	
    gl_FragColor = texture2D(texture, texCoordVarying) * colorVarying;
	
#endif
}

//------------------------------------------------------------------------------
