//
//  Shader_PreLitTexReflectiveWater.vsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 11/06/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

#define COMPRESSED_LIGHT_RATIOS

#define REFLECTION
//#define REFLECTION_LATLONG

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4	modelViewProjectionMatrix;
uniform highp	vec4	modelWorldOffset;

uniform highp	vec4	animUV;

#if defined (COMPRESSED_LIGHT_RATIOS)
uniform lowp	mat4	lightColorMat;
#endif

#if defined (REFLECTION)
uniform highp	mat4	modelViewMatrix;
#endif

//------------------------------------------------------------------------------

varying lowp	vec4	colorVarying;
varying highp	vec2	texCoordVarying;	// highp needed to avoid inaccuracies in the anim uv's

#if defined (REFLECTION)
varying highp	vec2	texCoordVarying2;
varying highp	vec2	texCoordVarying3;
varying mediump	vec3	eyeVectorVarying;
#endif

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER) + modelWorldOffset.xyz;
	vec4 posScaled4 = vec4(posScaled, 1.0);
	gl_Position = modelViewProjectionMatrix * posScaled4;
    //gl_Position = modelViewProjectionMatrix * position;

    // Decompress UVs
    //texCoordVarying = animUV.xy + texCoord;
	highp vec2 baseUVOffset = animUV.xy * animUV.zw;
    texCoordVarying = baseUVOffset + (texCoord/1024.0);

#if defined (COMPRESSED_LIGHT_RATIOS)
	//	highp float ratios = (position.w / 32768.0);
	//	mediump vec3 dots = fract(vec3(1.0, 32.0, 1024.0) * ratios);
	//	colorVarying = (lightColorMat * vec4(dots, 1.0));
	mediump vec3 dots = fract(vec3(1.0/32768.0, 32.0/32768.0, 1024.0/32768.0) * position.w);
	colorVarying = (lightColorMat * vec4(dots, 1.0));
#else
	lowp vec3 rgb = color.xyz/127.0;
	colorVarying = vec4(rgb, 1.0);
#endif
	
	//colorVarying *= vec4(1.0, 1.0, 0.8, 1.0);
	
#if defined (REFLECTION)
	vec3 posVS       = (modelViewMatrix * posScaled4).xyz;
	vec3 eyeVectorVS = normalize(posVS);
	
    texCoordVarying2 = (texCoordVarying * 4.0);
    texCoordVarying3 = (texCoord/1024.0) - baseUVOffset;
	
#if defined (REFLECTION_LATLONG)
	// Our tangent space is basically world space...
	mat3 viewTangentMatrix;
	viewTangentMatrix[0] = vec3(modelViewMatrix[0].x, modelViewMatrix[1].x, modelViewMatrix[2].x);
	viewTangentMatrix[1] = vec3(modelViewMatrix[0].y, modelViewMatrix[1].y, modelViewMatrix[2].y);
	viewTangentMatrix[2] = vec3(modelViewMatrix[0].z, modelViewMatrix[1].z, modelViewMatrix[2].z);
	
	eyeVectorVarying = (viewTangentMatrix * eyeVectorVS);
#else
	eyeVectorVarying = eyeVectorVS;
#endif	
#endif
}

//------------------------------------------------------------------------------
