//
//  Shader_PreLitTexReflectiveWater.vsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 11/06/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

#define COMPRESSED_LIGHT_RATIOS

#define REFLECTION

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4	modelViewProjectionMatrix;
uniform highp	vec4	modelWorldOffset;

uniform highp	vec4	animUV;

#if defined (COMPRESSED_LIGHT_RATIOS)
uniform lowp	mat4	lightColorMat;
#endif

#if defined (REFLECTION)
uniform highp	mat4	modelViewMatrix;
#endif

//------------------------------------------------------------------------------

varying lowp	vec4	colorVarying;
varying highp	vec2	texCoordVarying;	// highp needed to avoid inaccuracies in the anim uv's

#if defined (REFLECTION)
varying highp	vec2	texCoordVarying2;
varying mediump	vec3	reflectVectorVarying;
#endif

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER) + modelWorldOffset.xyz;
	vec4 posScaled4 = vec4(posScaled, 1.0);
	gl_Position = modelViewProjectionMatrix * posScaled4;
    //gl_Position = modelViewProjectionMatrix * position;
	
    // Decompress UVs
    //texCoordVarying = animUV.xy + texCoord;
	highp vec2 baseUVOffset = animUV.xy * animUV.zw;
    texCoordVarying = baseUVOffset + (texCoord/1024.0);

#if defined (COMPRESSED_LIGHT_RATIOS)
	//	highp float ratios = (position.w / 32768.0);
	//	mediump vec3 dots = fract(vec3(1.0, 32.0, 1024.0) * ratios);
	//	colorVarying = (lightColorMat * vec4(dots, 1.0));
	mediump vec3 dots = fract(vec3(1.0/32768.0, 32.0/32768.0, 1024.0/32768.0) * position.w);
	lowp	vec4 diffuseColor = (lightColorMat * vec4(dots, 1.0));
#else
	lowp	vec3 rgb = color.xyz/127.0;
	lowp	vec4 diffuseColor = vec4(rgb, 1.0);
#endif
	
	//colorVarying *= vec4(1.0, 1.0, 0.8, 1.0);
	
#if defined (REFLECTION)
	
    texCoordVarying2 = (texCoord/256.0) + (baseUVOffset * 3.0);

	// New fast approximation of reflection shader - calculate reflection vector in VS and then do a hacky distort in FS...
	vec3			posVS       = (modelViewMatrix * posScaled4).xyz;
	mediump vec3	eyeVectorVS = normalize(posVS);
	mediump	vec3	normal = vec3(0,0,1);
	
	reflectVectorVarying = reflect(eyeVectorVS, normal) * 32.0;
	
	colorVarying = (diffuseColor * 0.6);
#else
	colorVarying = diffuseColor;
#endif
}

//------------------------------------------------------------------------------
