//
//  Shader_PreLitTex.vsh
//  CityBuilder
//
//  Created by Jonathan Webb on 18/08/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

#define COMPRESSED_LIGHT_RATIOS

//------------------------------------------------------------------------------

attribute vec4 position;

#if !defined (COMPRESSED_LIGHT_RATIOS)
attribute vec4 color;
#endif

attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4	modelViewProjectionMatrix;
uniform highp	vec4	modelWorldOffset;

uniform highp	vec4	nightLightMapUVScaleOffset;

#if defined (COMPRESSED_LIGHT_RATIOS)
uniform lowp	mat4	lightColorMat;
#endif

//------------------------------------------------------------------------------

varying lowp	vec4	colorVarying;
varying highp	vec2	texCoordVarying;
varying highp	vec2	texCoordLightMapVarying;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec3 posScaled = (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER) + modelWorldOffset.xyz;
    gl_Position    = modelViewProjectionMatrix * vec4(posScaled, 1.0);
    //gl_Position = modelViewProjectionMatrix * position;

    // Decompress the UVs
    //texCoordVarying = texCoord;
    texCoordVarying = (texCoord/1024.0);

//	highp vec4 nightLightMapUVScaleOffset_local = vec4(0.00045904, 0.000536705, -623.235, -1830.38);
//	texCoordLightMapVarying.xy = ((posScaled.xz - nightLightMapUVScaleOffset_local.zw) * nightLightMapUVScaleOffset_local.xy);
	texCoordLightMapVarying.xy = ((posScaled.xz - nightLightMapUVScaleOffset.zw) * nightLightMapUVScaleOffset.xy);
	texCoordLightMapVarying.y = (1.0 - texCoordLightMapVarying.y);
	texCoordLightMapVarying.x += (1.5 / 2048.0);

#if defined (COMPRESSED_LIGHT_RATIOS)
//	highp float ratios = (position.w / 32768.0);
//	mediump vec3 dots = fract(vec3(1.0, 32.0, 1024.0) * ratios);
//	colorVarying = (lightColorMat * vec4(dots, 1.0));
	mediump vec3 dots = fract(vec3(1.0/32768.0, 32.0/32768.0, 1024.0/32768.0) * position.w);
	colorVarying = (lightColorMat * vec4(dots, 1.0));
#else
	lowp vec3 rgb = color.xyz/127.0;
	colorVarying = vec4(rgb, 1.0);
#endif
}

//------------------------------------------------------------------------------
