//
//  Shader_PreLitTexReflectiveWater.fsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 11/06/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define PI 3.1415926

#define NORMALMAP_FLATTEN_FACTOR	(2.0)

#define UW_RGB_SCALE	(0.7)
#define UW_RGB_SCALE_UP	(1.0/UW_RGB_SCALE)

//-----UW_RGB_SCALE_UP-------------------------------------------------------------------------

uniform sampler2D		normalmap;
uniform sampler2D		reflectionmap;
uniform sampler2D		refractionmap;

//------------------------------------------------------------------------------

varying mediump	vec3	varying_NormUV1_ReflectScale;
varying mediump	vec4	varying_RefractUV_NormUV2;
varying mediump	vec4	varying_EyeVector_RefractValue;
varying mediump	vec4	varying_LightDir_Roughness;

//------------------------------------------------------------------------------

void main()
{
	mediump	vec4	normalTexel		= texture2D(normalmap, varying_NormUV1_ReflectScale.xy);
	
	mediump	vec3	normal0			= ((2.0 * normalTexel.xyz) - 1.0);
	mediump	vec2	normal2UV		= (varying_RefractUV_NormUV2.zw + (normal0.yx * 0.05));
	mediump	vec3	normal2			= ((2.0 * texture2D(normalmap, normal2UV).xyz) - 1.0);
//	mediump	vec3	normal3			= normalize(vec3(normal2.x, (normal2.z * NORMALMAP_FLATTEN_FACTOR), -normal2.y));

	mediump float	roughness		= varying_LightDir_Roughness.w;

////					normal3			= normalize(normal3 + vec3(0.0, roughness, 0.0));
//	mediump	vec3	normal3			= normalize(vec3(normal2.x, ((normal2.z * NORMALMAP_FLATTEN_FACTOR) + roughness), -normal2.y));
	mediump	vec3	normal3			= normalize(vec3(normal2.y, ((normal2.z * NORMALMAP_FLATTEN_FACTOR) + roughness), -normal2.x));

//	mediump	vec3	normal3			= vec3(0.0, -1.0, 0.0);
	
	mediump float	eyeDotNormal	= dot(varying_EyeVector_RefractValue.xyz, normal3);

	mediump vec3	refVector		= normalize((2.0 * eyeDotNormal * normal3) - varying_EyeVector_RefractValue.xyz);

	mediump vec2	reflectionUV	= ((refVector.xz * 0.5) + 0.5);
	mediump vec4	reflection		= texture2D(reflectionmap, reflectionUV);

	mediump	vec2	refractedUV		= (varying_RefractUV_NormUV2.xy + (normal3.xz * varying_EyeVector_RefractValue.w));
	mediump	vec4	refraction		= texture2D(refractionmap, refractedUV);
	
	mediump	float	oneMinusDot		= (1.0 - eyeDotNormal);
	mediump	float	fresnel			= clamp(((1.8 * (oneMinusDot * oneMinusDot)) - 0.2), 0.0, 1.0);

	mediump	float	ndotl			= dot(varying_LightDir_Roughness.xyz, normal3);
//	mediump	float	light			= (0.7 + (0.3 * clamp(ndotl, 0.0, 1.0)));
	mediump	float	light			= clamp(ndotl, 0.0, 1.0);
	
//	mediump	vec3	specularRefVec	= normalize((2.0 * ndotl * normal3) - varying_LightDir_Roughness.xyz);
//	mediump	float	specular		= max(pow(dot(varying_EyeVector_RefractValue.xyz, specularRefVec), 64.0), 0.0);
	mediump	float	specular		= (fresnel * light);
					specular		= (specular * specular * 0.9);

	// Tone down the reflection and specular for now...
	fresnel  *= 0.5;
//	specular *= 0.75;

	mediump vec3	finalRefraction	= ((refraction.xyz * (UW_RGB_SCALE_UP * (0.5 + (light * 0.5)))));// * (1.0 - fresnel));
	mediump vec3	finalReflection	= (reflection.xyz * fresnel) * varying_NormUV1_ReflectScale.z;
	mediump vec3	finalSpecular	= vec3(specular, specular, specular) * fresnel;

//	gl_FragColor = vec4(roughness, roughness, roughness, 1.0);
//	gl_FragColor = vec4(varying_EyeVector_RefractValue.w, varying_EyeVector_RefractValue.w, varying_EyeVector_RefractValue.w, 1.0);

	gl_FragColor = vec4((finalRefraction + finalReflection + finalSpecular), 1.0);
}

//------------------------------------------------------------------------------
