//
//  Shader_PreLitTexReflectiveWater.vsh
//  CityBuilder
//
//  Created by Lee Metcalfe on 11/06/2011.
//  Copyright (c) 2011 Midoki Ltd. All rights reserved.
//

//------------------------------------------------------------------------------

#define POS_COMPRESSION_FACTOR				(16.0)	//(16.0)
#define POS_COMPRESSION_FACTOR_SKIN			(256.0)	//(16.0)

#define XYZ_DECOMPRESSION_MULTIPLIER		(1.0/POS_COMPRESSION_FACTOR)
#define XYZ_DECOMPRESSION_MULTIPLIER_SKIN	(1.0/POS_COMPRESSION_FACTOR_SKIN)

//------------------------------------------------------------------------------

#define NORMALMAP_UV_DIVISOR1	(256.0)	//(256.0)
#define NORMALMAP_UV_DIVISOR2	(500.0)	//(500.0)

#define REFRACT_DIST_MAX		(350.0)//(700.0)

#define REFRACT_OFFSET			(0.2)
#define REFRACT_SCALE			(0.8)
#define REFRACT_UV_MULTIPLIER	(0.05)//(0.025)

#define REFRACT_JOIN_SCALE		(0.3)

//------------------------------------------------------------------------------

attribute vec4 position;
attribute vec2 texCoord;

//------------------------------------------------------------------------------

uniform highp	mat4	modelViewProjectionMatrix;
uniform highp	mat4	modelMatrix;
uniform highp	vec4	animUV;
uniform highp	vec3	eyePosition;
uniform highp	mat4	lightDirectionMat;

//------------------------------------------------------------------------------

varying mediump	vec2	varying_NormUV1;
varying mediump	vec3	varying_WaveUV_EdgeRatio;
varying mediump	vec4	varying_RefractUV_NormUV2;
varying mediump	vec4	varying_EyeVector_RefractValue;
varying mediump	vec4	varying_LightDir_Roughness;

//------------------------------------------------------------------------------

void main()
{
    // Transform the position into screen space.
    vec3	posScaled			= (position.xyz * XYZ_DECOMPRESSION_MULTIPLIER);
	vec4	posScaled4			= vec4(posScaled, 1.0);
	vec4	posTrans			= modelViewProjectionMatrix * posScaled4;

	gl_Position							= posTrans;
	
	mediump	vec2	texCoordScreen		= vec2(-posScaled.x, posScaled.z) * 2.0;//3.0;//8.0;

	mediump vec2	texCoordScaled		= (texCoord.xy/1024.0);

	mediump vec2	baseUVOffset		= fract(animUV.xy * animUV.zw);
	mediump vec2	waveUVOffset		= fract(baseUVOffset * vec2(4.0, -6.0));
					
	mediump vec2	waveUV				= (texCoordScaled.xy + waveUVOffset);
	mediump float	waveEdgeRatio		= max(abs(texCoordScaled.y), 0.0);

	varying_WaveUV_EdgeRatio			= vec3(waveUV, waveEdgeRatio);

	mediump	vec2	normUV1				= ((texCoordScreen / NORMALMAP_UV_DIVISOR1) + baseUVOffset);
	mediump	vec2	normUV2				= ((texCoordScreen / NORMALMAP_UV_DIVISOR2) + (baseUVOffset * 2.0));
	mediump	vec2	refractUV			= (((posTrans.xy / posTrans.w) * 0.5) + 0.5);

    varying_NormUV1						= normUV1;
    varying_RefractUV_NormUV2			= vec4(refractUV, normUV2);
	
	mediump vec3	eyeToPos			= (posScaled.xyz - eyePosition.xyz);
	mediump float	eyeToPosDist		= length(eyeToPos);
	mediump	vec3	eyeVector			= normalize(eyeToPos / eyeToPosDist);

	mediump float	refractValueTemp	= max((1.0 - ((eyeToPosDist / REFRACT_DIST_MAX) / length(modelViewProjectionMatrix[1].xyz))), 0.0);
	mediump float	refractValue		= min((REFRACT_OFFSET + (refractValueTemp * REFRACT_SCALE)), 1.0);

	// Adjust the refraction to match the start of the water refraction...
					refractValue		= refractValue * (texCoordScaled.y * REFRACT_JOIN_SCALE);
	
	mediump float	roughness			= min(((refractValue * 0.7) + 0.3), 1.0);
	mediump float	vCoordRoughness		= 1.0;
					roughness			= max(((vCoordRoughness - roughness) * 1.8), 0.0);
		
	varying_EyeVector_RefractValue		= vec4(-eyeVector, (refractValue * REFRACT_UV_MULTIPLIER));
	varying_LightDir_Roughness			= vec4(lightDirectionMat[0].x, lightDirectionMat[1].x, lightDirectionMat[2].x, roughness);
	
//	varying_EyeVector_RefractValue		= vec4(-eyeVector, 0.0);
//	varying_LightDir_Roughness			= vec4(lightDirectionMat[0].x, lightDirectionMat[1].x, lightDirectionMat[2].x, 1.0);
}

//------------------------------------------------------------------------------
